@@ -1,6 +1,6 @@
 \dontrun{
 # using default filenames
-bps09 <- readBPS_2009(path = "C:/BPS/2009")
+bps09 <- readBPS_2009(path = "~/BPS/2009")
 dim(bps09)
 
 # parameters specified without default filenames
