@@ -1,6 +1,6 @@
 \dontrun{
 # the following call returns an edsurvey.data.frame to PIAAC for Canada
-can <- readPIAAC("C:/PIAAC", countries = "can")
+can <- readPIAAC("~/PIAAC/Cycle 1/", countries = "can")
 
 # extract a data.frame with a few variables
 gg <- getData(can, c("c_d05","ageg10lfs"))  
@@ -9,4 +9,3 @@ head(gg)
 # conduct an analysis on the edsurvey.data.frame
 edsurveyTable(~ c_d05 + ageg10lfs, data = can)
 }
-
