@@ -1,7 +1,17 @@
-es1REF <- c("", "Formula: composite ~ dsex + b017451 ", "", "Plausible values: 5", 
-            "jrrIMax: 1", "Weight variable: 'origwt'", "Variance method: jackknife", 
-            "JK replicates: 62", "full data n: 17606", "n used: 16331", "", 
-            "", "Summary Table:", "   dsex              b017451    N    WTD_N      PCT   SE(PCT)     MEAN SE(MEAN)", 
+es1REF <- c("",
+            "Formula: composite ~ dsex + b017451 ",
+            "",
+            "Plausible values: 5", 
+            "jrrIMax: 1",
+            "Weight variable: 'origwt'",
+            "Variance method: jackknife", 
+            "JK replicates: 62",
+            "full data n: 17606",
+            "n used: 16331",
+            "", 
+            "",
+            "Summary Table:",
+            "   dsex              b017451    N    WTD_N      PCT   SE(PCT)     MEAN SE(MEAN)", 
             "   Male Never or hardly ever 2350 2434.844 29.00978 0.6959418 270.8243 1.057078", 
             "   Male Once every few weeks 1603 1638.745 19.52472 0.5020657 275.0807 1.305922", 
             "   Male    About once a week 1384 1423.312 16.95795 0.5057265 281.5612 1.409587", 
@@ -512,7 +522,8 @@ gapPSUREF <- c("Call: gap(variable = \"composite\", data = sdf, groupA = dsex %i
                "Labels:",
                " group         definition nFullData nUsed nPSU", 
                "     A   dsex %in% \"Male\"     17606  8486  124",
-               "     B dsex %in% \"Female\"     17606  8429  124", "",
+               "     B dsex %in% \"Female\"     17606  8429  124",
+               "",
                "Percentage:",
                "     pctA    pctAse     pctB    pctBse    diffAB      covAB diffABse diffABpValue    dofAB", 
                " 50.27015 0.5016796 49.72985 0.5016796 0.5402935 -0.2516824 1.003359    0.5924778 53.45667", "",
@@ -555,12 +566,31 @@ sDiscreteREF <- c("Estimates are not weighted.",
                   "1   Male 8486   50.17", 
                   "2 Female 8429   49.83")
 
-cor_nocondenseREF <- c("Method: Pearson", "full data n: 17606", "n used: 14816", "", 
-                       "Correlation: 0.1783484", "", "Correlation Levels:", "  Levels for Variable 'c046501' (Lowest level first):", 
-                       "    1. 0%", "    2. 1-5%", "    3. 6-10%", "    4. 11-25%", 
-                       "    5. 26-50%", "    8. Over 90%", "  Levels for Variable 'c044006' (Lowest level first):", 
-                       "    1. None", "    2. 1-5%", "    3. 6-10%", "    4. 11-25%", 
-                       "    5. 26-50%", "    6. 51-75%", "    7. 76-90%", "    8. Over 90%"
+cor_nocondenseREF <- c("Method: Pearson",
+                       "full data n: 17606",
+                       "n used: 14816",
+                       "", 
+                       "Correlation: 0.1783",
+                       "Standard Error: 0.06575",
+                       "Confidence Interval: [0.03961, 0.3103]",
+                       "",
+                       "Correlation Levels:",
+                       "  Levels for Variable 'c046501' (Lowest level first):", 
+                       "    1. 0%",
+                       "    2. 1-5%",
+                       "    3. 6-10%",
+                       "    4. 11-25%", 
+                       "    5. 26-50%",
+                       "    8. Over 90%",
+                       "  Levels for Variable 'c044006' (Lowest level first):", 
+                       "    1. None",
+                       "    2. 1-5%",
+                       "    3. 6-10%",
+                       "    4. 11-25%", 
+                       "    5. 26-50%",
+                       "    6. 51-75%",
+                       "    7. 76-90%",
+                       "    8. Over 90%"
 )
 
 stdCoefREF <- c("                               coef     se       t   dof  Pr(>|t|)  stdCoef    stdSE", 
