@@ -23,8 +23,9 @@ if (!dir.exists(edsurveyHome)) {
  
 # 1. Check data read-in
 test_that("PISA data reads in correctly", {
-  expect_silent(downloadPISA(root=edsurveyHome, year=c(2003, 2006, 2009, 2012, 2015), cache=FALSE, verbose = FALSE))
+  expect_silent(downloadPISA(root=edsurveyHome, year=c(2003, 2006, 2009, 2012, 2015, 2018), cache=FALSE, verbose = FALSE))
   expect_silent(downloadPISA(root=edsurveyHome, year=2012, database="CBA", cache=FALSE, verbose = FALSE))
+  expect_silent(usaINT2018 <<- readPISA(paste0(edsurveyHome, "PISA/2018"), countries = "usa", verbose = FALSE))
   expect_silent(usaINT2015 <<- readPISA(paste0(edsurveyHome, "PISA/2015"), countries = "usa", verbose = FALSE))
   expect_silent(usaINT2012 <<- readPISA(paste0(edsurveyHome, "PISA/2012"), countries = "usa", verbose = FALSE))
   expect_silent(qcnCBA2012 <<- readPISA(paste0(edsurveyHome, "PISA/2012"), database = "CBA", countries = "qcn", verbose = FALSE)) # Shanghai
